#!/usr/bin/php
<?php

require_once('/usr/local/opmon/etc/config.php');

function main() {

	try {
		$opt = parse_options();
	} catch(Exception $e) {
		print sprintf("%s\n", $e->getMessage());
		return 3;
    }

	$countername = $opt->data_counter;

    $wmic = WmicFactory::createFromCredentialId($opt->host, $opt->credential);

	try {
		$ret = $wmic->do_query(
		    "select {$countername} ".
		    "from Win32_PerfFormattedData_Counters_ProcessorInformation ".
		    "where Name = '_Total'"
		);
	} catch(Exception $e) {
		print sprintf("%s\n", $e->getMessage());
		return 3;
	}

	if (count($ret) == 0) {
		print "Unable to retrieve CPU counters\n";
		return 3;
	}

	$ret = array_shift($ret);

	$ecode = 0;
	$outstr = NULL;
	foreach($ret->objects as $obj) {
		if ($obj->Name != "_Total")
			continue;

		if ($obj->$countername > $opt->critical) {
			$ecode = 2;
		} else if ($obj->$countername > $opt->warning) {
			$ecode = 1;
		}

		$outstr = sprintf("%s %s |%s=%s;%s;%s;0,100\n",
		    $countername,
		    $obj->$countername,
		    $countername,
		    $obj->$countername,
		    $opt->warning,
		    $opt->critical
		);
		break;
	}

	if (is_null($outstr)) {
		print "Unable to retrieve CPU counters\n";
		return 3;
	}

	print $outstr;
	return $ecode;
}

function parse_options()
{
    $opt = new StdClass();
    $opt->warning = 90;
    $opt->critical = 95;
    $opt->data_counter = "PercentProcessorTime";
    $options = getopt("h:D:C:w:c:d:");

    if (! isset($options['h'])) {
        throw new Exception("Invalid host address(-h)");
    }

    if (! isset($options['C'])) {
        throw new Exception("Invalid username(-C)");
    }

    if (isset($options['w'])) {
        $opt->warning = sprintf("%.2f", $options['w']);
    }

    if (isset($options['c'])) {
        $opt->critical = sprintf("%.2f", $options['c']);
    }

    if (isset($options['d'])) {
        $opt->data_counter = $options['d'];
    }

    $opt->host = $options['h'];
    $opt->domain = isset($options['D']) ? $options['D'] : "";
    $opt->credential = $options['C'];

    return $opt;
}

exit(main());
