#!/usr/bin/php
<?php

require_once('/usr/local/opmon/etc/config.php');

function main() {

    try {
        $opt = parse_options();
    } catch(Exception $e) {
        print sprintf("%s\n", $e->getMessage());
        return 3;
    }
    $cname = $opt->counter_name;
    $dname = $opt->disk_name;

    $wmic = WmicFactory::createFromCredentialId($opt->host, $opt->credential);

    try {
        $ret = $wmic->do_query(
            "select {$cname} ".
            "from Win32_PerfFormattedData_PerfDisk_PhysicalDisk ".
            "where Name='{$dname}'"
        );
    } catch(Exception $e) {
        print sprintf("%s\n", $e->getMessage());
        return 3;
    }

    if (count($ret) === 0) {
        print "Unable to locate disk {$dname}\n";
        return 3;
    }

    $ret = array_shift($ret);
    $obj = array_shift($ret->objects);
    $val = $obj->$cname;

    print sprintf("Disk %s %s %s|%s=%s;%s;%s;;\n",
        $dname,
        $cname,
        $val,
        $cname,
        $val,
        $opt->warning,
        $opt->critical
    );

    if ($opt->warning < $opt->critical) {
        if ($val > $opt->critical)
            return 2;
        if ($val > $opt->warning)
            return 1;
        return 0;
    }

    if ($val < $opt->critical)
        return 2;
    if ($val < $opt->warning)
        return 1;
    return 0;
}

function parse_options() {

    $opt = new StdClass();
    $opt->warning = 80;
    $opt->critical = 90;

    $options = getopt("h:D:C:n:d:w:c:");
    if (!isset($options['h']))
        throw new Exception("Invalid host address(-h)");
    if (!isset($options['C']))
        throw new Exception("Invalid credential(-C)");
    if (!isset($options['n']))
        throw new Exception("Invalid disk name(-n)");
    if (!isset($options['d']))
        throw new Exception("Invalid disk counter name(-d)");
    if (isset($options['w']))
        $opt->warning = sprintf("%d", $options['w']);
    if (isset($options['c']))
        $opt->critical = sprintf("%d", $options['c']);

    $opt->host = $options['h'];
    $opt->credential = $options['C'];
    $opt->disk_name = $options['n'];
    $opt->counter_name = $options['d'];
    $opt->domain = isset($options['D']) ? $options['D'] : "";

    return $opt;
}

exit(main());
