#!/usr/bin/php
<?php

require_once('/usr/local/opmon/etc/config.php');

function main() {

    $translation = array(
        "ForegroundApplicationBoost" => array(
            "None",
            "Minimum",
            "(Default) Maximum"
        ),
        "LargeSystemCache" => array(
            "Optimize memory for applications",
            "Optimize memory for system performance"
        ),
        "OperatingSystemSKU" => array(
            "0" => "Undefined",
            "1" => "Ultimate Edition",
            "2" => "Home Basic Edition",
            "3" => "Home Premium Edition",
            "4" => "Enterprise Edition",
            "5" => "Home Basic N Edition",
            "6" => "Business Edition",
            "7" => "Standard Server Edition",
            "8" => "Datacenter Server Edition",
            "9" => "Small Business Server Edition",
            "10" => "Enterprise Server Edition",
            "11" => "Starter Edition",
            "12" => "Datacenter Server Core Edition",
            "13" => "Standard Server Core Edition",
            "14" => "Enterprise Server Core Edition",
            "15" => "Enterprise Server Edition for Itanium-Based Systems",
            "16" => "Business N Edition",
            "17" => "Web Server Edition",
            "18" => "Cluster Server Edition",
            "19" => "Home Server Edition",
            "20" => "Storage Express Server Edition",
            "21" => "Storage Standard Server Edition",
            "22" => "Storage Workgroup Server Edition",
            "23" => "Storage Enterprise Server Edition",
            "24" => "Server For Small Business Edition",
            "25" => "Small Business Server Premium Edition",
            "26" => "Home Premium N",
            "27" => "Enterprise N",
            "28" => "Ultimate N",
            "29" => "Web Server, Server Core",
            "30" => "Windows Essential Business Server Management Server",
            "31" => "Windows Essential Business Server Security Server",
            "32" => "Windows Essential Business Server Messaging Server",
            "33" => "Server Foundation",
            "34" => "Windows Home Server 2011",
            "35" => "Windows Server 2008 without Hyper-V for Windows Essential Server Solutions",
            "36" => "Server Standard without Hyper-V",
            "37" => "Server Datacenter without Hyper-V (full installation)",
            "38" => "Server Enterprise without Hyper-V (full installation)",
            "39" => "Datacenter Edition without Hyper-V, Server Core",
            "40" => "Standard Edition without Hyper-V, Server Core",
            "41" => "Enterprise Edition without Hyper-V, Server Core",
            "42" => "Hyper-V Server",
            "43" => "Storage Server Express (core installation)",
            "44" => "Storage Server Standard (core installation)",
            "45" => "Storage Server Workgroup (core installation)",
            "46" => "Storage Server Enterprise (core installation)",
            "47" => "Starter N",
            "48" => "Professional",
            "49" => "Professional N",
            "50" => "Windows Small Business Server 2011 Essentials",
            "51" => "Server For SB Solutions",
            "52" => "Server Solutions Premium",
            "53" => "Server Solutions Premium (core installation)",
            "54" => "Server For SB Solutions EM",
            "55" => "Server For SB Solutions EM",
            "56" => "Windows MultiPoint Server",
            "59" => "Windows Essential Server Solution Management",
            "60" => "Windows Essential Server Solution Additional",
            "61" => "Windows Essential Server Solution Management SVC",
            "62" => "Windows Essential Server Solution Additional SVC",
            "63" => "Small Business Server Premium (core installation)",
            "64" => "Server Hyper Core V",
            "66" => "Starter E",
            "67" => "Home Basic E",
            "68" => "Home Premium E",
            "69" => "Professional E",
            "70" => "Enterprise E",
            "71" => "Ultimate E",
            "72" => "Server Enterprise (evaluation installation)",
            "76" => "Windows MultiPoint Server Standard (full installation)",
            "77" => "Windows MultiPoint Server Premium (full installation)",
            "79" => "Server Standard (evaluation installation)",
            "80" => "Server Datacenter (evaluation installation)",
            "84" => "Enterprise N (evaluation installation)",
            "95" => "Storage Server Workgroup (evaluation installation)",
            "96" => "Storage Server Standard (evaluation installation)",
            "98" => "Windows 8 N",
            "99" => "Windows 8 China",
            "100" => "Windows 8 Single Language",
            "101" => "Windows 8",
            "103" => "Professional with Media Center"
        )
    );

    $validate_by_content = array(
        "Status" => "OK"
    );

    $suite_masks = array(
        "1" => "Small Business",
        "2" => "Enterprise",
        "4" => "BackOffice",
        "8" => "Communications",
        "16" => "Terminal",
        "32" => "Small Business Restricted",
        "64" => "Embedded NT",
        "128" => "Data Center",
        "256" => "Single User",
        "512" => "Personal",
        "1024" => "Blade"
    );

    $show_only_counters = array(
        "BuildNumber",
        "CodeSet",
        "CountryCode",
        "EncryptionLevel",
        "Locale",
        "MaxNumberOfProcesses",
        "MaxProcessMemorySize",
        "OSArchitecture",
        "OSType",
        "PAEEnabled",
        "SerialNumber",
        "SystemDirectory",
        "Version",
        "WindowsDirectory",
        "TotalVisibleMemorySize"
    );


    try {
        $opt = parse_options();
    } catch(Exception $e) {
        print sprintf("%s\n", $e->getMessage());
        return 3;
    }
    $cname = $opt->counter_name;

    $wmic = WmicFactory::createFromCredentialId($opt->host, $opt->credential);

    try {
        $ret = $wmic->do_query(
            "select {$cname} from Win32_OPeratingSystem"
        );
    } catch(Exception $e) {
        print sprintf("%s\n", $e->getMessage());
        return 3;
    }

    if (count($ret) == 0) {
        print "Unable to locate counter {$cname}\n";
        return 3;
    }
    $ret = array_shift($ret);

    if (count($ret->objects) == 0) {
        print "Unable to locate counter {$cname}\n";
        return 3;
    }
    $ret = array_shift($ret->objects);
    $val = $ret->$cname;

    /* exceptions */
    if (in_array($cname, $show_only_counters)) {
        print sprintf("%s %s\n", $cname, $val);
        return 0;
    }

    if (isset($validate_by_content[$cname])) {
        print sprintf("%s %s\n", $cname, $val);
        if ($val == $validate_by_content[$cname])
            return 0;
        return 2;
    }

    if ($cname == "SuiteMask") {
        $f = FALSE;
        foreach($suite_masks as $id => $descr) {
            if ($id & $val) {
                print "$descr ";
                $f = TRUE;
            }
        }

        if ($f == FALSE) {
            print "Unknown suite mask $val\n";
            return 3;
        }

        print "\n";
        return 0;
    }

    if (isset($translation[$cname])) {
        $tt = $translation[$cname];

        if (!isset($tt[$val])) {
            print sprintf("%s %s(unknown index)\n", $cname, $val);
            return 3;
        }

        print sprintf("%s: %s\n", $cname, $tt[$val]);
        return 0;
    }

    print sprintf("%s %s|%s=%s;%s;%s;;\n",
        $cname,
        $val,
        $cname,
        $val,
        $opt->warning,
        $opt->critical
    );

    if ($opt->warning < $opt->critical) {
        if ($val > $opt->critical)
            return 2;
        if ($val > $opt->warning)
            return 1;
        return 0;
    }

    if ($val < $opt->critical)
        return 2;
    if ($val < $opt->warning)
        return 1;
    return 0;
}

function parse_options() {

    $opt = new StdClass();
    $opt->warning = 80;
    $opt->critical = 90;

    $options = getopt("h:D:C:d:w:c:");
    if (! isset($options['h']))
        throw new Exception("Invalid host address(-h)");
    if (! isset($options['C']))
        throw new Exception("Invalid credential(-C)");
    if (! isset($options['d']))
        throw new Exception("Invalid counter name(-d)");
    if (isset($options['w']))
        $opt->warning = sprintf("%d",$options['w']);
    if (isset($options['c']))
        $opt->critical = sprintf("%d",$options['c']);

    $opt->host = $options['h'];
    $opt->credential = $options['C'];
    $opt->counter_name = $options['d'];
    $opt->domain = isset($options['D']) ? $options['D'] : "";


    return $opt;
}

exit(main());
