#!/usr/bin/php
<?php

require_once('/usr/local/opmon/etc/config.php');

function main() {

    try {
        $opt = parse_options();
    } catch(Exception $e) {
        print sprintf("%s\n", $e->getMessage());
        return 3;
    }
    $fname = $opt->page_file;

    $wmic = WmicFactory::createFromCredentialId($opt->host, $opt->credential);

    try {
        $ret = $wmic->do_query("select * from Win32_PageFileUsage");
    } catch(Exception $e) {
        print sprintf("%s\n", $e->getMessage());
        return 3;
    }

    if (count($ret) == 0) {
        print "Unable to locate paging file {$fname}\n";
        return 2;
    }
    $ret = array_shift($ret);

    $dt = NULL;
    foreach($ret->objects as $obj) {

        if ($obj->Caption !== $fname)
            continue;

        $dt = $obj;
        break;
    }

    if (is_null($dt)) {
        print "Unable to paging file {$fname}\n";
        return 2;
    }

    $outstr = sprintf("%s usage: %sMBs |usage=%sMBs;%s;%s;;\n",
        $fname,
        $dt->CurrentUsage,
        $dt->CurrentUsage,
        $opt->warning,
        $opt->critical
    );

    print $outstr;
    if ($dt->CurrentUsage > $opt->critical)
        return 2;
    else if ($dt->CurrentUsage > $opt->warning)
        return 1;
    return 0;
}

function parse_options() {

    $opt = new StdClass();
    $opt->warning = NULL;
    $opt->critical = NULL;

    $options = getopt("h:C:w:c:d:D:");
    if (!isset($options['h']))
        throw new Exception("Invalid host address(-h)");
    if (!isset($options['C']))
        throw new Exception("Invalid credential(-C)");
    if (!isset($options['d']))
        throw new Exception("Invalid page file caption(-d)");
    if (isset($options['w']))
        $opt->warning = sprintf("%d", $options['w']);
    if (isset($options['c']))
        $opt->critical = sprintf("%d", $options['c']);

    $opt->host = $options['h'];
    $opt->credential = $options['C'];
    $opt->page_file = $options['d'];
    $opt->domain = isset($options['D']) ? $options['D'] : "";

    return $opt;
}

exit(main());

