<?php

class WMIManager
{

    private function get_wmi_obj($data)
    {
        if (! isset($data->globalcredential)) {
            throw new Exception("Credential id is missing");
        }

        if (! isset($data->globalhost)) {
            throw new Exception("Host address id is missing");
        }

        return WmicFactory::createFromCredentialId(
            $data->globalhost,
            $data->globalcredential
        );
    }

    public function get_processes($data)
    {
        $wmi = $this->get_wmi_obj($data);
        $ret = $wmi->do_query(
            "select * from Win32_Process"
        );

        if (count($ret) === 0) {
            throw new Exception("No process");
        }

        $ret = array_shift($ret);

        $json = array();
        $already_seen = array();
        foreach ($ret->objects as $obj) {
            if (in_array($obj->Caption, $already_seen)) {
                continue;
            }

            $already_seen[] = $obj->Caption;
            $json[] = array('key' => $obj->Caption, 'value' => $obj->Caption);
        }

        return $json;
    }

    public function get_services($data)
    {
        $wmi = $this->get_wmi_obj($data);

        $ret = $wmi->do_query("select Name,Started from Win32_Service");

        if (count($ret) === 0) {
            throw new Exception(
                "No service returned on query: 'select Name,Started from Win32_Service'"
            );
        }

        $ret = array_shift($ret);

        $json = array();
        foreach ($ret->objects as $obj) {
            if ($obj->Started != 'True') {
                continue;
            }

            $json[] = array('key' => $obj->Name, 'value' => $obj->Name);
        }

        return $json;

    }

    public function get_pagefiles($data)
    {
        $wmi = $this->get_wmi_obj($data);

        $ret = $wmi->do_query("select * from Win32_PageFileUsage");

        if (count($ret) === 0) {
            throw new Exception("No paging files");
        }

        $ret = array_shift($ret);

        $json = array();
        $seen = array();
        foreach ($ret->objects as $obj) {
            if (in_array($obj->Caption, $seen)) {
                continue;
            }

            $seen[] = $obj->Caption;
            $json[] = array('key' => $obj->Caption, 'value' => $obj->Caption);
        }

        return $json;
    }

    public function get_processes_counters($data)
    {
        $ignored_counters = array(
            "ParentProcessId",
            "ProcessId",
            "SessionId",
            "WindowsVersion",
            "TerminationDate",
            "Status",
            "ExecutionState"
        );

        $wmi = $this->get_wmi_obj($data);

        $ret = $wmi->do_query("select * from Win32_Process");

        if (count($ret) === 0) {
            throw new Exception("No process");
        }

        $ret = array_shift($ret);

        $json = array();
        $proc = array_shift($ret->objects);
        foreach ($proc as $key => $value) {
            if (ctype_digit($value) === false) {
                continue;
            }

            if (in_array($key, $ignored_counters)) {
                continue;
            }

            $json[] = array('key' => $key, 'value' => $key);
        }

        return $json;
    }

    public function get_disks($data)
    {
        $wmi = $this->get_wmi_obj($data);

        $ret = $wmi->do_query(
            "select Name from Win32_PerfFormattedData_PerfDisk_PhysicalDisk"
        );

        if (count($ret) === 0) {
            throw new Exception(
                "Empty return on 'select Name from " .
                "Win32_PerfFormattedData_PerfDisk_PhysicalDisk'"
            );
        }

        $ret = array_shift($ret);

        $json = array();
        foreach ($ret->objects as $obj) {
            $json[] = array('key' => $obj->Name, 'value' => $obj->Name);
        }

        return $json;
    }

    public function get_disk_counters($data)
    {
        $wmi = $this->get_wmi_obj($data);

        try {
            $ret = $wmi->do_query("
                SELECT
                    *
                FROM
                    Win32_PerfFormattedData_PerfDisk_PhysicalDisk
                WHERE
                    Name='_Total'
            ");
        } catch (Exception $e) {
            return array();
        }


        if (count($ret) === 0) {
            return array();
        }

        $ret = array_shift($ret);

        foreach ($ret->objects as $obj) {
            if ($obj->Name != "_Total") {
                continue;
            }

            $json = array();
            foreach ($obj as $key => $value) {
                if (ctype_digit($value) === false) {
                    continue;
                }

                $json[] = array('key' => $key, 'value' => $key);
            }
            return $json;
        }

        return array();

    }

    public function get_os_counters($data)
    {
        $ignored_counters = array("Organization");

        $wmi = $this->get_wmi_obj($data);

        $ret = $wmi->do_query("select * from Win32_OPeratingSystem");

        if (count($ret) === 0) {
            throw new Exception("Unable to retrieve counters\n");
        }

        $ret = array_shift($ret);

        if (count($ret->objects) === 0) {
            return array();
        }

        $sample = array_shift($ret->objects);
        $json = array();
        foreach ($sample as $k => $v) {
            if (ctype_digit($v) === false) {
                continue;
            }

            if (in_array($key, $ignored_counters)) {
                continue;
            }

            $json[] = array('key' => $k, 'value' => $k);
        }

        return $json;
    }

    public function get_processor_counters($data)
    {
        $ignored_counters = array("Name");

        $wmi = $this->get_wmi_obj($data);

        $ret = $wmi->do_query(
            "select * from Win32_PerfFormattedData_Counters_ProcessorInformation"
        );

        if (count($ret) === 0) {
            throw new Exception("Unable to retrieve counters\n");
        }

        $ret = array_shift($ret);

        foreach ($ret->objects as $obj) {
            if ($obj->Name != "_Total") {
                continue;
            }

            $json = array();
            foreach ($obj as $key => $value) {
                if (in_array($key, $ignored_counters)) {
                    continue;
                }

                $json[] = array('key' => $key, 'value' => $key);
            }
            return $json;
        }

        return array();
    }
}
